﻿//---------------------------------------------------------------------------
// Copyright © Victor Chen, http://www.cppfans.com/

#include <vcl.h>
#pragma hdrstop

#include "VictorMediaPlayer.h"
#include "UnitVmpMain.h"
#include "UnitVmpFullScreen.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormVmpMain *FormVmpMain;
//---------------------------------------------------------------------------
__fastcall TFormVmpMain::TFormVmpMain(TComponent* Owner)
    : TForm(Owner)
{
  _bChangingPos = false;
  LabelPos->Caption = L"";

  LabelLAV->Caption = L"";
  LabelLAV->ShowAccelChar = false;
  LabelLAV->Font->Color = clLime;
  LabelLAV->Color = clBlack;
  LabelLAV->Transparent = false;
  LabelLAV->Visible = false;

  ButtonPlay->Enabled = true;
  ButtonPause->Enabled = false;
  ButtonStop->Enabled = false;

  OpenDialog1->Filter =
   L"视频 (*.mkv, *.ts, *.mp4, *.webm, *.avi, *.qt, *.mov, *.mpg, *.mpeg, *.m1v, *.asf, *.wmv)" L"|"
         L"*.mkv;*.ts;*.mp4;*.webm;*.avi;*.qt;*.mov;*.mpg;*.mpeg;*.m1v;*.asf;*.wmv"             L"|"
   L"音频 (*.aac, *.mp3, *.mpa, *.mp2, *.wma, *.wav, *.au, *.aif, *.aiff, *.snd)"               L"|"
         L"*.aac;*.mp3;*.mpa;*.mp2;*.wma;*.wav;*.au;*.aif;*.aiff;*.snd"                         L"|"
   L"MIDI (*.mid, *.midi, *.rmi)"                                                               L"|"
         L"*.mid;*.midi;*.rmi"                                                                  L"|"
   L"图片 (*.jpg, *.bmp, *.gif, *.tga)"                                                         L"|"
         L"*.jpg;*.bmp;*.gif;*.tga"                                                             L"|"
   L"所有 (*.*)"                                                                                L"|"
         L"*.*"                                                                                   ;

  vmp = new TVictorMediaPlayer;
  vmp->OnStateChanged = VmpStateChanged;
  vmp->hWndVideo = PanelVideo->Handle;

  CheckBoxAspRet->Checked = vmp->AspectRatio;
}
//---------------------------------------------------------------------------
__fastcall TFormVmpMain::~TFormVmpMain()
{
  delete vmp;
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::VmpStateChanged(TVictorMediaPlayer *Sender, TVictorMediaPlayState NewSt, TVictorMediaPlayState OldSt)
{
  ButtonPlay->Enabled = NewSt != vmpsRunning && NewSt != vmpsPaused;
  ButtonPause->Enabled = NewSt == vmpsRunning || NewSt == vmpsPaused;
  ButtonStop->Enabled = NewSt == vmpsRunning || NewSt == vmpsPaused;
  ButtonPause->Caption = NewSt == vmpsPaused ? L"继续" : L"暂停";
  Edit1->Enabled = NewSt != vmpsRunning && NewSt != vmpsPaused;
  ButtonBrowse->Enabled = NewSt != vmpsRunning && NewSt != vmpsPaused;

  double d = Sender->Duration;
  if(d>0)
   {
     double p = Sender->Position;
     TrackBarPos->Position = 100.0*p/d + 0.5;
     LabelPos->Caption = FormatDateTime(L"hh:nn:ss",p/86400.0) + L"/"+ FormatDateTime(L"hh:nn:ss", d/86400.0);
   }

  UnicodeString sLavInfo;
  if(vmp->LavSplitterOK)
   {
     if(vmp->LavVideoOK && vmp->LavAudioOK)
       sLavInfo = L"LAV Video & Audio";
     else if(vmp->LavVideoOK)
       sLavInfo = L"LAV Video";
     else if(vmp->LavAudioOK)
       sLavInfo = L"LAV Audio";
     else
       sLavInfo = L"LAV";
   }
  else
   {
     sLavInfo = L"No LAV";
   }
  LabelLAV->Caption = sLavInfo;
  if(!LabelLAV->Visible)
    LabelLAV->Visible = true;
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::ButtonPlayClick(TObject *Sender)
{
  try
   {
     if(vmp->PlayState != vmpsPaused)
       vmp->FileName = Edit1->Text;
     vmp->Play();
   }
  catch(Exception &e)
   {
     MessageBox(Handle,e.Message.c_str(),Caption.c_str(),MB_OK|MB_ICONSTOP);
   }
  if(!vmp->LavSplitterOK)
   {
     UnicodeString sMsg =
      L"加载 LAV 解码器失败!\r\n"
      L"请使用 regsvr32 注册 LAV 的以下三个文件:\r\n"
      L"LAVSplitter.ax, LAVVideo.ax, LAVAudio.ax,\r\n"
      L"就可以使用 LAV 解码器播放视频和音频了。"
      ;
     MessageBox(Handle, sMsg.c_str(), Caption.c_str(), MB_OK|MB_ICONSTOP);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::ButtonPauseClick(TObject *Sender)
{
  if(vmp->PlayState==vmpsPaused)
    vmp->Play();
  else
    vmp->Pause();
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::ButtonStopClick(TObject *Sender)
{
  vmp->Stop();
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::ButtonBrowseClick(TObject *Sender)
{
  if(OpenDialog1->Execute(Handle))
   {
     Edit1->Text = OpenDialog1->FileName;
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::PanelVideoResize(TObject *Sender)
{
  vmp->AdjVideoSize();
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::CheckBoxAspRetClick(TObject *Sender)
{
  vmp->AspectRatio = CheckBoxAspRet->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::ButtonFullScrClick(TObject *Sender)
{
  ShowFullScreen();
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::MiFullScrClick(TObject *Sender)
{
  ShowFullScreen();
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::PanelVideoDblClick(TObject *Sender)
{
  ShowFullScreen();
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::Timer1Timer(TObject *Sender)
{
  if(vmp->PlayState==vmpsRunning)
   {
     try
      {
        _bChangingPos = true;

        double d = vmp->Duration;
        if(d>0)
         {
           double p = vmp->Position;
           TrackBarPos->Position = 100.0*p/d + 0.5;
           LabelPos->Caption = FormatDateTime(L"hh:nn:ss",p/86400.0) + L"/"+ FormatDateTime(L"hh:nn:ss", d/86400.0);
         }
      }
     __finally
      {
        _bChangingPos = false;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::TrackBarPosChange(TObject *Sender)
{
  if(!_bChangingPos)
   {
     try
      {
        _bChangingPos = true;

        if(vmp->PlayState==vmpsRunning)
         {
           double d = vmp->Duration;
           if(d>0)
            {
              vmp->Position = TrackBarPos->Position*d/100.0;
            }
         }
      }
     __finally
      {
        _bChangingPos = false;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormVmpMain::ShowFullScreen(void)
{
  if(!_bFormVmpFullScreenCreated)
   {
     FormVmpFullScreen = new TFormVmpFullScreen(this);
     FormVmpFullScreen->Left = 0;
     FormVmpFullScreen->Top = 0;
     FormVmpFullScreen->Width = Screen->Width;
     FormVmpFullScreen->Height = Screen->Height;
     FormVmpFullScreen->Show();
     FormVmpFullScreen->SetVmp(vmp);
   }
}
//---------------------------------------------------------------------------
